package com.github.jfcloud.excel.editor.docdeal.config;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.*;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestAttributes;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.Objects;
import java.util.concurrent.TimeUnit;

/**
 * 接口日志切面配置
 *
 * @author zj
 * @date 2022/6/7
 **/
@Aspect
@Component
@Order(0)
@Slf4j
@RequiredArgsConstructor
public class LogAspect {

    /**
     * 操作版本号
     */
    private final String operVer ="ONLY.12";

    /**
     * 设置web日志切入点
     */
    @Pointcut("execution(public * com.github.jfcloud.excel.editor.docdeal.controller.*.*(..))")
    public void webLog() {
    }

    @Before("webLog()")
    public void deBefore(JoinPoint joinPoint) {
        // 接收到请求，记录请求内容
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = Objects.requireNonNull(attributes).getRequest();
        // 记录下请求内容
        log.info("=======================================================================");
        log.info("VERSION : " + operVer);
        log.info("URL : " + request.getRequestURL().toString());
        log.info("HTTP_METHOD : " + request.getMethod());
        //网关代理，Ip始终是zuul网关ip
        log.info("IP : " + request.getRemoteAddr());
        log.info("方法调用位置 : " + joinPoint.getSignature().getDeclaringTypeName() + "." + joinPoint.getSignature().getName());
        String param = Arrays.toString(joinPoint.getArgs());
        log.info("参数 : " + param);
    }

    @AfterReturning(returning = "ret", pointcut = "webLog()")
    public void doAfterReturning(Object ret) {
        // 处理完请求，返回内容
        log.info("方法的返回值 : " + JSON.toJSONString(ret));
    }

}