package com.github.jfcloud.excel.editor.docdeal.oss.http;

import com.amazonaws.services.s3.model.Bucket;
import com.amazonaws.services.s3.model.S3Object;
import com.amazonaws.services.s3.model.S3ObjectSummary;
import com.github.jfcloud.excel.editor.docdeal.oss.service.OssTemplate;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author zj
 * @date 2023-4-25
 */
@RestController
@RequestMapping({"/oss"})
public class OssEndpoint {
    private final OssTemplate template;

    @PostMapping({"/bucket/{bucketName}"})
    public Bucket createBucker(@PathVariable String bucketName) {
        this.template.createBucket(bucketName);
        return this.template.getBucket(bucketName).get();
    }

    @GetMapping({"/bucket"})
    public List<Bucket> getBuckets() {
        return this.template.getAllBuckets();
    }

    @GetMapping({"/bucket/{bucketName}"})
    public Bucket getBucket(@PathVariable String bucketName) {
        return this.template.getBucket(bucketName).orElseThrow(() -> new IllegalArgumentException("Bucket Name not found!"));
    }

    @DeleteMapping({"/bucket/{bucketName}"})
    @ResponseStatus(HttpStatus.ACCEPTED)
    public void deleteBucket(@PathVariable String bucketName) {
        this.template.removeBucket(bucketName);
    }

    @PostMapping({"/object/{bucketName}"})
    public S3Object createObject(@RequestBody MultipartFile object, @PathVariable String bucketName) throws Exception {
        String name = object.getOriginalFilename();
        this.template.putObject(bucketName, name, object.getInputStream(), object.getSize(), object.getContentType());
        return this.template.getObjectInfo(bucketName, name);
    }

    @PostMapping({"/object/{bucketName}/{objectName}"})
    public S3Object createObject(@RequestBody MultipartFile object, @PathVariable String bucketName, @PathVariable String objectName) throws Exception {
        this.template.putObject(bucketName, objectName, object.getInputStream(), object.getSize(), object.getContentType());
        return this.template.getObjectInfo(bucketName, objectName);
    }

    @GetMapping({"/object/{bucketName}/{objectName}"})
    public List<S3ObjectSummary> filterObject(@PathVariable String bucketName, @PathVariable String objectName) {
        return this.template.getAllObjectsByPrefix(bucketName, objectName, true);
    }

    @GetMapping({"/object/{bucketName}/{objectName}/{expires}"})
    public Map<String, Object> getObject(@PathVariable String bucketName, @PathVariable String objectName, @PathVariable Integer expires) {
        Map<String, Object> responseBody = new HashMap(8);
        responseBody.put("bucket", bucketName);
        responseBody.put("object", objectName);
        responseBody.put("url", this.template.getObjectURL(bucketName, objectName, expires));
        responseBody.put("expires", expires);
        return responseBody;
    }

    @ResponseStatus(HttpStatus.ACCEPTED)
    @DeleteMapping({"/object/{bucketName}/{objectName}/"})
    public void deleteObject(@PathVariable String bucketName, @PathVariable String objectName) throws Exception {
        this.template.removeObject(bucketName, objectName);
    }

    public OssEndpoint(OssTemplate template) {
        this.template = template;
    }
}
